/*
 * Created on Jul 17, 2003
 *
 * To change the template for this generated file go to
 * Window>Preferences>Java>Code Generation>Code and Comments
 */
package gov.grants.xmlToDB.dao;

import java.util.*;
import java.sql.*;
import gov.grants.xmlToDB.valueobject.*;


/**
 * @author dxia
 *
 * Sf424header Data Access Object (DAO).
 * This class contains all database handling that is needed to 
 * permanently store and retrieve Sf424header object instances. 
 */


public class Sf424headerDao {


	/**
	 * load-method. This will load valueObject contents from database using
	 * Primary-Key as identifier. Upper layer should use this so that valueObject
	 * instance is created and only primary-key should be specified. Then call
	 * this method to complete other persistent information. This method will
	 * overwrite all other fields except primary-key and possible runtime variables.
	 *
	 * @param conn         This method requires working database connection.
	 * @param valueObject  This parameter contains the class instance to be loaded.
	 *                     Primary-key field must be set for this to work properly.
	 */
	public static void load(Connection conn, Sf424header valueObject) throws SQLException {

		  if (valueObject.getGrants_govTrackingNumber() == null) {
			   //System.out.println("Can not select without Primary-Key!");
			   throw new SQLException("Can not select without Primary-Key!");
		  }
		  String sql = "SELECT * FROM Sf424header WHERE Grants_govTrackingNumber = ?"; 
		  PreparedStatement stmt = null;

		  try {
			   stmt = conn.prepareStatement(sql);

			   stmt.setString(1, valueObject.getGrants_govTrackingNumber());
			   singleQuery(conn, stmt, valueObject);

		  } finally {
			  if (stmt != null)
				  stmt.close();
		  }
	}


	/**
	 * LoadAll-method. This will read all contents from database table and
	 * build an Vector containing valueObjects. Please note, that this method
	 * will consume huge amounts of resources if table has lot's of rows. 
	 * This should only be used when target tables have only small amounts
	 * of data.
	 *
	 * @param conn         This method requires working database connection.
	 */
	public static List loadAll(Connection conn) throws SQLException {

		  List searchResults;

		  String sql = "SELECT * FROM Sf424header ";
		  sql = sql + "ORDER BY Grants_govTrackingNumber ";

		  searchResults = listQuery(conn, conn.prepareStatement(sql));


		  return searchResults;
	}



	/**
	 * create-method. This will create new row in database according to supplied
	 * valueObject contents. Make sure that values for all NOT NULL columns are
	 * correctly specified. Also, if this table does not use automatic surrogate-keys
	 * the primary-key must be specified. After INSERT command this method will 
	 * read the generated primary-key back to valueObject if automatic surrogate-keys
	 * were used. 
	 *
	 * @param conn         This method requires working database connection.
	 * @param valueObject  This parameter contains the class instance to be created.
	 *                     If automatic surrogate-keys are not used the Primary-key 
	 *                     field must be set for this to work properly.
	 */
	public static void create(Connection conn, Sf424header valueObject) throws SQLException {

		  String sql = "INSERT INTO Sf424header ( Grants_govTrackingNumber, AgencyName, CFDANumber, ActivityTitle" + 
		  ", OpportunityID, OpportunityTitle, CompetitionID" + 
		  ", OpeningDate, ClosingDate, SubmissionTitle" + 
		  ", HashValue, HashAlgorithm, SchemaVersion) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? )"; 
		  PreparedStatement stmt = null;

		  try {
			   stmt = conn.prepareStatement(sql);

			   stmt.setString(1, valueObject.getGrants_govTrackingNumber());
			   stmt.setString(2, valueObject.getAgencyName());
			   stmt.setString(3, valueObject.getCFDANumber());
			   stmt.setString(4, valueObject.getActivityTitle());
			   stmt.setString(5, valueObject.getOpportunityID());
			   stmt.setString(6, valueObject.getOpportunityTitle());
			   stmt.setString(7, valueObject.getCompetitionID());
			   stmt.setDate(8, valueObject.getOpeningDate());
			   stmt.setDate(9, valueObject.getClosingDate());
			   stmt.setString(10, valueObject.getSubmissionTitle());
			   stmt.setString(11, valueObject.getHashValue());
			   stmt.setString(12, valueObject.getHashAlgorithm());
			   stmt.setString(13, valueObject.getSchemaVersion());
			   int rowcount = databaseUpdate(conn, stmt);
			   if (rowcount != 1) {
					//System.out.println("PrimaryKey Error when updating DB!");
					throw new SQLException("PrimaryKey Error when updating DB!");
			   }

		  } finally {
			  if (stmt != null)
				  stmt.close();
		  }



	}


	/**
	 * save-method. This method will save the current state of valueObject to database.
	 * Save can not be used to create new instances in database, so upper layer must
	 * make sure that the primary-key is correctly specified. Primary-key will indicate
	 * which instance is going to be updated in database.
	 *
	 * @param conn         This method requires working database connection.
	 * @param valueObject  This parameter contains the class instance to be saved.
	 *                     Primary-key field must be set for this to work properly.
	 */
	public static void save(Connection conn, Sf424header valueObject) throws SQLException {

		  String sql = "UPDATE Sf424header SET Grants_govTrackingNumber = ?, AgencyName = ?, CFDANumber = ?" + 
		  ", ActivityTitle = ?, OpportunityID = ?, OpportunityTitle = ?" + 
		  ", CompetitionID = ?, OpeningDate = ?, ClosingDate = ?" + 
		  ", SubmissionTitle = ?, HashValue = ?, HashAlgorithm = ?" + 
		  ", SchemaVersion = ? WHERE Grants_govTrackingNumber = ?"; 
		  PreparedStatement stmt = null;

		  try {
			  stmt = conn.prepareStatement(sql);

			  stmt.setString(1, valueObject.getGrants_govTrackingNumber());
			  stmt.setString(2, valueObject.getAgencyName());
			  stmt.setString(3, valueObject.getCFDANumber());
			  stmt.setString(4, valueObject.getActivityTitle());
			  stmt.setString(5, valueObject.getOpportunityID());
			  stmt.setString(6, valueObject.getOpportunityTitle());
			  stmt.setString(7, valueObject.getCompetitionID());
			  stmt.setDate(8, valueObject.getOpeningDate());
			  stmt.setDate(9, valueObject.getClosingDate());
			  stmt.setString(10, valueObject.getSubmissionTitle());
			  stmt.setString(11, valueObject.getHashValue());
			  stmt.setString(12, valueObject.getHashAlgorithm());
			  stmt.setString(13, valueObject.getSchemaVersion());
			  stmt.setString(14, valueObject.getGrants_govTrackingNumber());

			  int rowcount = databaseUpdate(conn, stmt);
			  if (rowcount != 1) {
				   //System.out.println("PrimaryKey Error when updating DB!");
				   throw new SQLException("PrimaryKey Error when updating DB!");
			  }
		  } finally {
			  if (stmt != null)
				  stmt.close();
		  }
	}


	/**
	 * delete-method. This method will remove the information from database as identified by
	 * by primary-key in supplied valueObject. Once valueObject has been deleted it can not 
	 * be restored by calling save. Restoring can only be done using create method but if 
	 * database is using automatic surrogate-keys, the resulting object will have different 
	 * primary-key than what it was in the deleted object.
	 *
	 * @param conn         This method requires working database connection.
	 * @param valueObject  This parameter contains the class instance to be deleted.
	 *                     Primary-key field must be set for this to work properly.
	 */
	public static void delete(Connection conn, Sf424header valueObject) throws SQLException {

		  if (valueObject.getGrants_govTrackingNumber() == null) {
			   //System.out.println("Can not delete without Primary-Key!");
			   throw new SQLException("Can not delete without Primary-Key!");
		  }
		  String sql = "DELETE FROM Sf424header WHERE Grants_govTrackingNumber = ?";
		  PreparedStatement stmt = null;

		  try {
			  stmt = conn.prepareStatement(sql);

			   stmt.setString(1, valueObject.getGrants_govTrackingNumber());

			  int rowcount = databaseUpdate(conn, stmt);
			  if (rowcount != 1) {
				   //System.out.println("PrimaryKey Error when updating DB!");
				   throw new SQLException("PrimaryKey Error when updating DB!");
			  }
		  } finally {
			  if (stmt != null)
				  stmt.close();
		  }
	}


	/**
	 * deleteAll-method. This method will remove all information from the table that matches
	 * this Dao and ValueObject couple. This should be the most efficient way to clear table.
	 * Once deleteAll has been called, no valueObject that has been created before can be 
	 * restored by calling save. Restoring can only be done using create method but if database 
	 * is using automatic surrogate-keys, the resulting object will have different primary-key 
	 * than what it was in the deleted object. (Note, the implementation of this method should
	 * be different with different DB backends.)
	 *
	 * @param conn         This method requires working database connection.
	 */
	public static void deleteAll(Connection conn) throws SQLException {

		  String sql = "DELETE FROM Sf424header";
		  PreparedStatement stmt = null;

		  try {
			  stmt = conn.prepareStatement(sql);
			  int rowcount = databaseUpdate(conn, stmt);
		  } finally {
			  if (stmt != null)
				  stmt.close();
		  }
	}


	/**
	 * coutAll-method. This method will return the number of all rows from table that matches
	 * this Dao. The implementation will simply execute "select count(primarykey) from table".
	 * If table is empty, the return value is 0. This method should be used before calling
	 * loadAll, to make sure table has not too many rows.
	 *
	 * @param conn         This method requires working database connection.
	 */
	public static int countAll(Connection conn) throws SQLException {

		  String sql = "SELECT count(Grants_govTrackingNumber) FROM Sf424header";
		  PreparedStatement stmt = null;
		  ResultSet result = null;
		  int allRows = 0;

		  try {
			  stmt = conn.prepareStatement(sql);
			  result = stmt.executeQuery();

			  if (result.next())
				  allRows = result.getInt(1);
		  } finally {
			  if (result != null)
				  result.close();
			  if (stmt != null)
				  stmt.close();
		  }
		  return allRows;
	}


	/** 
	 * searchMatching-Method. This method provides searching capability to 
	 * get matching valueObjects from database. It works by searching all 
	 * objects that match permanent instance variables of given object.
	 * Upper layer should use this by setting some parameters in valueObject
	 * and then  call searchMatching. The result will be 0-N objects in vector, 
	 * all matching those criteria you specified. Those instance-variables that
	 * have NULL values are excluded in search-criteria.
	 *
	 * @param conn         This method requires working database connection.
	 * @param valueObject  This parameter contains the class instance where search will be based.
	 *                     Primary-key field should not be set.
	 */
	public static List searchMatching(Connection conn, Sf424header valueObject) throws SQLException {

		  List searchResults;

		  boolean first = true;
		  StringBuffer sql = new StringBuffer("SELECT * FROM Sf424header WHERE 1=1 ");
		  if (valueObject.getGrants_govTrackingNumber() != null) {
			  if (first) { first = false; }
			  sql.append("AND Grants_govTrackingNumber LIKE '").append(valueObject.getGrants_govTrackingNumber()).append("%' ");
		  }

		  if (valueObject.getAgencyName() != null) {
			  if (first) { first = false; }
			  sql.append("AND AgencyName LIKE '").append(valueObject.getAgencyName()).append("%' ");
		  }

		  if (valueObject.getCFDANumber() != null) {
			  if (first) { first = false; }
			  sql.append("AND CFDANumber LIKE '").append(valueObject.getCFDANumber()).append("%' ");
		  }

		  if (valueObject.getActivityTitle() != null) {
			  if (first) { first = false; }
			  sql.append("AND ActivityTitle LIKE '").append(valueObject.getActivityTitle()).append("%' ");
		  }

		  if (valueObject.getOpportunityID() != null) {
			  if (first) { first = false; }
			  sql.append("AND OpportunityID LIKE '").append(valueObject.getOpportunityID()).append("%' ");
		  }

		  if (valueObject.getOpportunityTitle() != null) {
			  if (first) { first = false; }
			  sql.append("AND OpportunityTitle LIKE '").append(valueObject.getOpportunityTitle()).append("%' ");
		  }

		  if (valueObject.getCompetitionID() != null) {
			  if (first) { first = false; }
			  sql.append("AND CompetitionID LIKE '").append(valueObject.getCompetitionID()).append("%' ");
		  }

		  if (valueObject.getOpeningDate() != null) {
			  if (first) { first = false; }
			  sql.append("AND OpeningDate = '").append(valueObject.getOpeningDate()).append("' ");
		  }

		  if (valueObject.getClosingDate() != null) {
			  if (first) { first = false; }
			  sql.append("AND ClosingDate = '").append(valueObject.getClosingDate()).append("' ");
		  }

		  if (valueObject.getSubmissionTitle() != null) {
			  if (first) { first = false; }
			  sql.append("AND SubmissionTitle LIKE '").append(valueObject.getSubmissionTitle()).append("%' ");
		  }

		  if (valueObject.getHashValue() != null) {
			  if (first) { first = false; }
			  sql.append("AND HashValue LIKE '").append(valueObject.getHashValue()).append("%' ");
		  }

		  if (valueObject.getHashAlgorithm() != null) {
			  if (first) { first = false; }
			  sql.append("AND HashAlgorithm LIKE '").append(valueObject.getHashAlgorithm()).append("%' ");
		  }

		  if (valueObject.getSchemaVersion() != null) {
			  if (first) { first = false; }
			  sql.append("AND SchemaVersion LIKE '").append(valueObject.getSchemaVersion()).append("%' ");
		  }

		  sql.append("ORDER BY Grants_govTrackingNumber ");

		  // Prevent accidential full table results.
		  // Use loadAll if all rows must be returned.
		  if (first)
			   searchResults = new Vector();
		  else
			   searchResults = listQuery(conn, conn.prepareStatement(sql.toString()));

		  return searchResults;
	}

	/**
	 * databaseUpdate-method. This method is a helper method for internal use. It will execute
	 * all database handling that will change the information in tables. SELECT queries will
	 * not be executed here however. The return value indicates how many rows were affected.
	 * This method will also make sure that if cache is used, it will reset when data changes.
	 *
	 * @param conn         This method requires working database connection.
	 * @param stmt         This parameter contains the SQL statement to be excuted.
	 */
	protected static int databaseUpdate(Connection conn, PreparedStatement stmt) throws SQLException {

		  int result = stmt.executeUpdate();
		  return result;
	}



	/**
	 * databaseQuery-method. This method is a helper method for internal use. It will execute
	 * all database queries that will return only one row. The resultset will be converted
	 * to valueObject. If no rows were found, an exception will be thrown.
	 *
	 * @param conn         This method requires working database connection.
	 * @param stmt         This parameter contains the SQL statement to be excuted.
	 * @param valueObject  Class-instance where resulting data will be stored.
	 */
	protected static void singleQuery(Connection conn, PreparedStatement stmt, Sf424header valueObject) throws SQLException {

		  ResultSet result = null;

		  try {
			  result = stmt.executeQuery();

			  if (result.next()) {
				   valueObject.setGrants_govTrackingNumber(result.getString("Grants_govTrackingNumber"));
				   valueObject.setAgencyName(result.getString("AgencyName"));
				   valueObject.setCFDANumber(result.getString("CFDANumber"));
				   valueObject.setActivityTitle(result.getString("ActivityTitle"));
				   valueObject.setOpportunityID(result.getString("OpportunityID"));
				   valueObject.setOpportunityTitle(result.getString("OpportunityTitle"));
				   valueObject.setCompetitionID(result.getString("CompetitionID"));
				   valueObject.setOpeningDate(result.getDate("OpeningDate"));
				   valueObject.setClosingDate(result.getDate("ClosingDate"));
				   valueObject.setSubmissionTitle(result.getString("SubmissionTitle"));
				   valueObject.setHashValue(result.getString("HashValue"));
				   valueObject.setHashAlgorithm(result.getString("HashAlgorithm"));
				   valueObject.setSchemaVersion(result.getString("SchemaVersion"));
			  } else {
					//System.out.println("Sf424header Object Not Found!");
					throw new SQLException("Sf424header Object Not Found!");
			  }
		  } finally {
			  if (result != null)
				  result.close();
			  if (stmt != null)
				  stmt.close();
		  }
	}


	/**
	 * databaseQuery-method. This method is a helper method for internal use. It will execute
	 * all database queries that will return multiple rows. The resultset will be converted
	 * to the List of valueObjects. If no rows were found, an empty List will be returned.
	 *
	 * @param conn         This method requires working database connection.
	 * @param stmt         This parameter contains the SQL statement to be excuted.
	 */
	protected static List listQuery(Connection conn, PreparedStatement stmt) throws SQLException {

		  Vector searchResults = new Vector();
		  ResultSet result = null;

		  try {
			  result = stmt.executeQuery();

			  while (result.next()) {
				   Sf424header temp = new Sf424header();
				   temp.setGrants_govTrackingNumber(result.getString("Grants_govTrackingNumber"));
				   temp.setAgencyName(result.getString("AgencyName"));
				   temp.setCFDANumber(result.getString("CFDANumber"));
				   temp.setActivityTitle(result.getString("ActivityTitle"));
				   temp.setOpportunityID(result.getString("OpportunityID"));
				   temp.setOpportunityTitle(result.getString("OpportunityTitle"));
				   temp.setCompetitionID(result.getString("CompetitionID"));
				   temp.setOpeningDate(result.getDate("OpeningDate"));
				   temp.setClosingDate(result.getDate("ClosingDate"));
				   temp.setSubmissionTitle(result.getString("SubmissionTitle"));
				   temp.setHashValue(result.getString("HashValue"));
				   temp.setHashAlgorithm(result.getString("HashAlgorithm"));
				   temp.setSchemaVersion(result.getString("SchemaVersion"));
				   searchResults.add(temp);
			  }

		  } finally {
			  if (result != null)
				  result.close();
			  if (stmt != null)
				  stmt.close();
		  }

		  return (List)searchResults;
	}

}

