/*
 *  GrantApplicationProcessor.java
 *
 *  @author :: Dan Dumitru ::
 *  Created on 2003-07-14
 *	
 *	This class unmarshals the application XML file components into JAXB 
 *  generated objects.  These objects are made available to other 
 *  application components via getter() methods
 * 
 *  This class is based upon the sample class presented in the
 *  tutorial "Data binding with JAXB" presented by developerWorks.
 *  Just go to http://www.ibm.com/developerWorks and search for
 *  "Data binding with JAXB"
 * 
 */
 
 package gov.grants.xmlToDB.utility;

// sun
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.File;

// jwsdp - sun
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

// reference implementation
import gov.grants.system.header.*;
import gov.grants.sf424.*;
import gov.grants.sf424a.*;
import gov.grants.sf424b.*;
import gov.grants.sf424c.*;
import gov.grants.sf424d.*;
import gov.grants.system.footer.*;

public class GrantApplicationProcessor {
	
	private String headerSystemPackageName = "gov.grants.system.header";
	private String sf424PackageName = "gov.grants.sf424";
	private String sf424aPackageName = "gov.grants.sf424a";
	private String sf424bPackageName = "gov.grants.sf424b";
	private String sf424cPackageName = "gov.grants.sf424c";
	private String sf424dPackageName = "gov.grants.sf424d";
	private String footerSystemPackageName = "gov.grants.system.footer";
	
	private String defaultXmlFileName = "GrantApplication00000062.xml";
	private JAXBContext jaxbContext;
	private Unmarshaller unmarshaller;
	private File applicationFile;
	
        private gov.grants.system.header.GrantSubmissionHeader header;
	private gov.grants.sf424.AwardingAgencyGrantApplication sf424GrantApplication;
	private gov.grants.sf424a.BudgetInformation sf424aBudgetInformation;
	private gov.grants.sf424b.Assurances sf424bAssurances;
	private gov.grants.sf424c.BudgetInformation sf424cBudgetInformation;
	private gov.grants.sf424d.Assurances sf424dAssurances;
        private gov.grants.system.footer.GrantSubmissionFooter footer;
	
	// CONSTRUCTORS
	
	GrantApplicationProcessor() {
		try {
			createApplication( this.defaultXmlFileName );
			debugApplication();
		} catch ( Exception e ) {
			;// do nothing - we have already reported the error via System.out.println();
		}
	}
	
	public GrantApplicationProcessor( String uploadedFileName ) throws Exception {
		this.defaultXmlFileName = uploadedFileName;
		createApplication( uploadedFileName );
		debugApplication();
	}
	
	// GETTERS
	
	// Header
	public gov.grants.system.header.GrantSubmissionHeader getGrantSubmissionHeader() {
		return this.header;
	}
	
	// SF424 (AKA, Core, Cover)
	public gov.grants.sf424.AwardingAgencyGrantApplication getSf424GrantApplication() {
		return this.sf424GrantApplication;
	}

	// SF424A
	public gov.grants.sf424a.BudgetInformation getSf424aBudgetInformation() {
		return this.sf424aBudgetInformation;
	}
	
	// SF424B
	public gov.grants.sf424b.Assurances getSf424bAssurances() {
		return this.sf424bAssurances;
	}
	
	// SF424C
	public gov.grants.sf424c.BudgetInformation getSf424cBudgetInformation() {
		return this.sf424cBudgetInformation;
	}
	
	// SF424D
	public gov.grants.sf424d.Assurances getSf424dAssurances() {
		return this.sf424dAssurances;
	}
	
	// Footer
	public gov.grants.system.footer.GrantSubmissionFooter getGrantSubmissionFooter() {
		return this.footer;
	}

	private void createApplication( String xmlFileName ) throws Exception {
		try {
			
			applicationFile = createFile( xmlFileName );
			
			// If desired, can plug in some other XML parser here, 
			// such as SAX or JDOM, if wish to avoid use of Java 
			// XML Binding (JAXB) techniques
			unmarshalFile( applicationFile );
			
		} catch (JAXBException e) {
			System.out.println( "There has been a problem either creating the " +
								"context for one of the packages, creating an " +
								"unmarshaller for it, or unmarshalling the '" +
								xmlFileName + "' file.\n\nFormally, the problem is a " + e );
			throw new Exception( "There has been a problem either creating the " +
								"context for one of the packages, creating an " +
								"unmarshaller for it, or unmarshalling the '" +
								xmlFileName + "' file.<br><br>Formally, the problem is a " + e );
								
		} catch ( FileNotFoundException e ) {
			System.out.println("There has been a problem locating the '" +
			xmlFileName + "' file.\n\nFormally, the problem is a " + e );
			throw new Exception( "There has been a problem locating the '" +
			xmlFileName + "' file.<br><br>Formally, the problem is a " + e );
		} catch ( Exception e ) {
			System.out.println("There has been a problem. Formally, the problem is a " + e );
			throw new Exception( "There has been a problem. Formally, the problem is a " + e ); 
		}
	}
	
	private JAXBContext createContext( String packageName ) throws JAXBException {
		return JAXBContext.newInstance( packageName );
	}
	
	private Unmarshaller createUnmarshaller( JAXBContext jaxbContext) throws JAXBException {
		Unmarshaller unmarshaller = jaxbContext.createUnmarshaller();
		unmarshaller.setValidating( true );
		
		return unmarshaller;
	}
	
	private File createFile( String xmlFileName ) throws FileNotFoundException {
		File file = new File( xmlFileName );
		if ( !file.exists() ) { 
			throw new FileNotFoundException();
		}
		return file;
	}
	
	private void unmarshalFile( File applicationFile ) 
	throws JAXBException, ParserConfigurationException, SAXException, IOException  {
		
		// No schema currently exists that defines the entire application.  
		// As a result, have to first parse the application file to retrieve the 
		// first-level elements under the document root.  These elements, which
		// represent the Application components (header, footer, etc) are then
		// sent to the JAXB unmarshaler 
		Node node;
		NodeList nodeList;
		
		DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
		docBuilderFactory.setNamespaceAware(true);
		DocumentBuilder docBuilder = docBuilderFactory.newDocumentBuilder();
		Document applicationDoc = docBuilder.parse( applicationFile );
		System.out.println( "Got applicationDoc" );
		
		// Not necessarily the most appropriate way of processing the elements.
		// Alternatively, retrieve each element individually by name
		nodeList = applicationDoc.getElementsByTagName( "*" );
		System.out.println( "Got nodeList" );
		
		if ( nodeList != null ) {
			for ( int i = 0; i < nodeList.getLength(); i++ ) {
				node = (Node) nodeList.item(i);
				
				// can refactor logic here to remove conditional statements
				if ( node.getNodeName().equalsIgnoreCase( "sys:GrantSubmissionHeader" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( headerSystemPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					header = ( GrantSubmissionHeader ) unmarshaller.unmarshal( node );
					System.out.println( "Got sys:GrantSubmissionHeader" );
//				} else if ( node.getNodeName().equalsIgnoreCase( "sf424:GrantApplication" ) ) {
//					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
//					jaxbContext = createContext( sf424PackageName );
//					unmarshaller = createUnmarshaller( jaxbContext );
//					sf424GrantApplication = ( GrantApplication ) unmarshaller.unmarshal( node );
//					System.out.println( "Got sf424:GrantApplication" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sf424:AwardingAgencyGrantApplication" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( sf424PackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					sf424GrantApplication = ( AwardingAgencyGrantApplication ) unmarshaller.unmarshal( node );
					System.out.println( "Got sf424:GrantApplication" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sf424a:BudgetInformation" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( sf424aPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					sf424aBudgetInformation = ( gov.grants.sf424a.BudgetInformation ) unmarshaller.unmarshal( node );
					System.out.println( "Got sf424a:BudgetInformation" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sf424b:Assurances" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( sf424bPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					sf424bAssurances= ( gov.grants.sf424b.Assurances ) unmarshaller.unmarshal( node );
					System.out.println( "Got sf424b:Assurances" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sf424c:BudgetInformation" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( sf424cPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					sf424cBudgetInformation = ( gov.grants.sf424c.BudgetInformation ) unmarshaller.unmarshal( node );
					System.out.println( "Got sf424c:BudgetInformation" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sf424d:BudgetInformation" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( sf424dPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					sf424dAssurances= ( gov.grants.sf424d.Assurances ) unmarshaller.unmarshal( node );
					System.out.println( "Got sf424d:Assurances" );
				} else if ( node.getNodeName().equalsIgnoreCase( "sys:GrantSubmissionFooter" ) ) {
					System.out.println( "Processing Node " + node.getNodeName() + "\n" );
					jaxbContext = createContext( footerSystemPackageName );
					unmarshaller = createUnmarshaller( jaxbContext );
					footer = ( GrantSubmissionFooter ) unmarshaller.unmarshal( node );
					System.out.println( "Got sys:GrantSubmissionFooter" );
				}
			}
		} else {
			System.out.println( "nodeList is null" );
		}
	}
	
   /** Note: to be really useful, an appropiate toString() method
	 * should be added to the JAXB generated objects to be debugged;
	 * i.e., add a relevant toString() method to the desired
	 * *TypeImpl classes defined in the gov.grants.sf424*.impl folders;
     */ 
	private void debugApplication() {
		System.out.println( header );
		System.out.println( sf424GrantApplication );
		System.out.println( sf424aBudgetInformation );
		System.out.println( footer );
	}
	
   /**
     * Use for 'batch', or standalone processing
     */
	public static void main( String[] args ) {
		String xmlFilename = "";
		try {
			if ( args[0] != null ) {
				xmlFilename = args[0];
				new GrantApplicationProcessor( xmlFilename );
			} else {
				new GrantApplicationProcessor();
			}
		} catch ( Exception e ) {
			System.out.println( "There has been a problem starting up the " +
						"GrantApplicationProcessor using filename " + 
						xmlFilename + 
						".\n\nFormally, the problem is a " + e );
		}
	}
}
