package gov.grants.xmlToDB.utility;

// sun
import java.io.*;
import java.util.*;
import javax.servlet.*;

public class MultipartRequest {
	class UploadedFile {
		private String dir;
		private String filename;
		private String type;
		
		UploadedFile(String dir,String filename,String type) {
			this.dir=dir;
			this.filename=filename;
			this.type=type;
			
		}
		
		public String getContentType() {
			return type;
		}
		
		public String getFilesystemName() {
			return filename;
		}
		
		
		
		public File getFile() {
			if(dir == null || filename == null) {
				return null;
			}
			else {
				return new File(dir + File.separator + filename);
			}
		}
	}

	class MultipartInputStreamHandler {
		
		ServletInputStream in;
		String boundary;
		int totalExpected;
		int totalRead=0;
		byte[] buf = new byte[8*1024];
		
		public MultipartInputStreamHandler(ServletInputStream in,String boundary,int totalExpected) {
			this.in=in;
			this.boundary=boundary;
			this.totalExpected=totalExpected;
		}
		
		public String readLine() throws IOException {
			StringBuffer sbuf=new StringBuffer();
			int result;
			String line;
			
			
			do {
				result=this.readLine(buf,0,buf.length);
				if (result != -1){
					sbuf.append(new String(buf,0,result,"ISO-8859-1"));
				}
				
			}while(result == buf.length);
			
			if(sbuf.length() == 0 ) {
				return null;
			}
			
			sbuf.setLength(sbuf.length() - 2);
			return sbuf.toString();
		}
		
		public int readLine(byte b[],int off,int len) throws IOException {
			if(totalRead >= totalExpected) {
				return -1;
			}
			
			else {
				int result=in.readLine(b,off,len);
				if(result > 0) {
					totalRead += result;
				}
				return result;
			}
		}
	}
	
	private static final int DEFAULT_MAX_POST_SIZE = 1024 * 1024;
	
	private ServletRequest req;
	private File dir;
	private int maxSize;
	
	private Hashtable parameters = new Hashtable();
	private Hashtable files = new Hashtable();
	
	public MultipartRequest(ServletRequest request,
		String saveDirectory) throws IOException {
		
		this(request,saveDirectory,DEFAULT_MAX_POST_SIZE);
		
	}
	
	
	public MultipartRequest(ServletRequest request,
		String saveDirectory,
		int maxPostSize) throws IOException {
		
		//sanity check values
		
		if(request == null)
			throw new IllegalArgumentException(" request is null");
		
		if(saveDirectory == null)
			throw new IllegalArgumentException("saveDirectory is null");
		
		if(maxPostSize <= 0)
			throw new IllegalArgumentException("maxPostsize must be positive");
		
		
		//save the request,dir and maxsize
		
		req=request;
		dir=new File(saveDirectory);
		maxSize=maxPostSize;
		
		//check saveDirectory is truely a directory
		
		if(!dir.isDirectory())
			throw new IllegalArgumentException("not a directory: " + saveDirectory);
		
		//check saveDirectory is writable
		
		if(!dir.canWrite())
			throw new IllegalArgumentException("not writable:" + saveDirectory );
		
		//now parse the request saving data to "parameters" and "files";
		//write the file contents to saveDirectory
		
		readRequest();
		
	}
	
	
	public Enumeration getParameterNames() {
		return parameters.keys();
	}
	
	
	public Enumeration getFileNames() {
		return files.keys();
	}
	
	
	public String getParameter(String name) {
		
		try{
			
			String param = (String)parameters.get(name);
			if(param.equals(""))return null;
			return param;
		}
		
		catch(Exception e) {
			return null;
		}
	}
	
	
	
	public String getFilesystemName(String name) {
		
		try {
			
			UploadedFile file = (UploadedFile)files.get(name);
			return file.getFilesystemName();
		}
		
		catch(Exception e) {
			return null;
		}
	}
	
	
	
	public String getContentType(String name) {
		try {
			
			UploadedFile file = (UploadedFile)files.get(name);
			return file.getContentType();
		}
		
		catch(Exception e) {
			return null;
		}
	}
	
	
	public File getFile(String name) {
		try {
			
			UploadedFile file=(UploadedFile)files.get(name);
			return file.getFile();
		}
		catch(Exception e) {
			return null;
		}
	}
	
	
	
	protected void readRequest() throws IOException {
		//check the content type to make sure it's "multipart/form-data"
		String type=req.getContentType();
		if(type == null ||
			!type.toLowerCase().startsWith("multipart/form-data")) {
			throw new IOException("posted content type isn't multipart/form-data");
		}
		
		
		//check the content length to prevent denial of service attacks
		int length = req.getContentLength();
		
		if(length > maxSize) {
			throw new IOException("posted content length of " + length +
				"exceeds limit of " + maxSize);
		}
		
		
		//get the boundary string ;it's included in the content type.
		//should look something like "------------------------12012133613061"
		
		String boundary=extractBoundary(type);
		
		if(boundary == null) {
			throw new IOException("seperation boundary was not specified");
		}
		
		//construct the special input stream we'll read from
		MultipartInputStreamHandler in = new MultipartInputStreamHandler(req.getInputStream(),boundary,length);
		
		//read the first line ,should be the first boundary
		String line =in.readLine();
		
		if(line == null) {
			throw new IOException("corrupt form data: premature ending");
		}
		
		//verify that the line is the boundary
		if(!line.startsWith(boundary)) {
			throw new IOException("corrupt form data: no leading boundary");
			
		}
		
		
		//now that we are just beyond the first boundary, loop over each part
		
		boolean done=false;
		
		while(!done) {
			done=readNextPart(in,boundary);
		}
	}
	
	
	
	protected boolean readNextPart(MultipartInputStreamHandler in,String boundary) throws IOException {
		//read the first line ,should look like this
		//content-disposition:form-data;name="field1";filename="file1.txt"
		
		String line=in.readLine();
		
		if(line == null) {
			//no parts left,we'r done
			return true;
		}
		
		
		//parse the content-disposition line
		
		String[] dispInfo=extractDispositionInfo(line);
		String disposition=dispInfo[0];
		String name=dispInfo[1];
		String filename=dispInfo[2];
		
		
		//now onto the next line.this will either be empty
		//or contain a content-type and an empty line.
		
		line=in.readLine();
		
		if(line == null) {
			//no parts left,we/r done
			return true;
		}
		
		
		//get the content type.or null if none is specified
		String contentType=extractContentType(line);
		
		if(contentType != null) {
			//eaat the empty line
			line=in.readLine();
			
			if(line == null||line.length() > 0) {
				//line should be empty
				throw new IOException("malformed line after content type: " + line);
			}
		}
		else {
			//assume a default content type
			contentType = "application/octet-stream";
		}
		
		//now,finally we read the content (end after reading the boundary)
		
		if(filename == null) {
			//this is a parameter
			String value=readParameter(in,boundary);
			parameters.put(name,value);
		}
		else {
			//this is a file
			filename = System.currentTimeMillis() + "_" + filename;
			readAndSaveFile(in,boundary,filename);
			
			if(filename.equals("unknown")) {
				files.put(name,new UploadedFile(null,null,null));
			}
			else {
				files.put(name,new UploadedFile(dir.toString(),filename,contentType));
			}
		}
		return false;
	}
	
	
	
	protected String readParameter(MultipartInputStreamHandler in,String boundary) throws IOException {
		StringBuffer sbuf = new StringBuffer();
		String line;
		
		
		while((line = in.readLine()) != null) {
			
			if(line.startsWith(boundary)) break;
			sbuf.append(line + "\r\n");
			//add the \r\n in case there are many lines
		}
		
		
		if(sbuf.length() == 0) {
			return null;
		}
		
		sbuf.setLength(sbuf.length() - 2);//cut off the last line's \r\n
		
		return sbuf.toString();  //no URL decoding is needed
	}
	
	
	protected void readAndSaveFile(MultipartInputStreamHandler in,String boundary,String filename) throws IOException {
		File f=new File(dir + File.separator + filename);
		FileOutputStream fos=new FileOutputStream(f);
		BufferedOutputStream out=new BufferedOutputStream(fos,8 * 1024);
		
		byte[]  bbuf=new byte[8 * 1024];
		int result;
		String line;
		
		//ServletInputSteam.readLine() has the annoying habit of
		//adding a \r\n to the end of the last line
		//since we want a byte-for-byte transfer,we have to cut those chars
		
		boolean rnflag= false;
		
		while((result=in.readLine(bbuf,0,bbuf.length)) != -1 ) {
			//check for the boundary
			
			if(result > 2 && bbuf[0] == '-' && bbuf[1] == '-') {
				line=new String(bbuf,0,result,"ISO-8859-1");
				
				if(line.startsWith(boundary))break;
			}
			
			
			//are we suppose to write \r\n for the last iteration ?
			
			if(rnflag) {
				out.write('\r');
				out.write('\n');
				rnflag=false;
			}
			
			//write the buffer ,postpone any editing \r\n
			
			if(result >= 2 && bbuf[result - 2 ] == '\r' &&
				bbuf[result - 1] == '\n' ) {
				out.write(bbuf,0,result - 2);
				rnflag=true;
			}
			else
			{
				out.write(bbuf,0,result);
			}
		}
		out.flush();
		out.close();
		fos.close();
	}
	
	private String extractBoundary(String line){
		int index=line.indexOf("boundary=");
		if (index==-1)
		{
			return null;
		}
		String boundary =line.substring(index+9);
		boundary="--"+boundary;
		return boundary;
	}
	
	private String[] extractDispositionInfo(String line) throws IOException {
		String[] retval=new String[3];
		String origline=line;
		line=origline.toLowerCase();
		
		int start =line.indexOf("content-disposition");
		int end =line.indexOf(";");
		if (start==-1||end==-1){
			throw new IOException("cont disposition corrupt" + origline);
			
		}
		String disposition=line.substring(start+21,end);
		if (!disposition.equals("form-data"))
		{
			throw new IOException ("invalid content disposition"+disposition);
		}
		
		start=line.indexOf("name=\"",end);
		end=line.indexOf("\"",start+7);
		if (start==-1||end==-1)
		{
			throw new IOException("content dispostion corrupt"+origline);
			
		}
		String name=origline.substring(start+6,end);
		String filename=null;
		start =line.indexOf("filename=\"",end+2);
		end=line.indexOf("\"",start+10);
		if (start!=-1 && end!=-1){
			filename=origline.substring(start+10,end);
			int slash=Math.max(filename.lastIndexOf('/'),filename.lastIndexOf('\\'));
			if (slash >-1){
				filename=filename.substring(slash+1);
			}
			if (filename.equals("")) filename="unknown";
		}
		retval[0]=disposition;
		retval[1]=name;
		retval[2]=filename;
		return retval;
	}
	
	
	
	private String extractContentType(String line) throws IOException {
		
		String contentType=null;
		//convert the line to a lowercase string
		String origline = line;
		line = origline.toLowerCase();
		
		if(line.startsWith("content-type")) {
			int start=line.indexOf(" ");
			
			if(start == -1) {
				throw new IOException("content type corrupt: " + origline);
			}
			
			contentType=line.substring(start + 1);
		}
		
		else if(line.length() != 0) {
			//no content typt,so should be empty
			throw new IOException("malformed line after disposition:" + origline);
		}
		return contentType;
	}
}





//A class to hold information about an uploaded file




//A class to aid in reading multipart/form-data from a ServletInputStream
//It keeps track of how many bytes have been read & detects when the
//content-Length limit has been reached .this is necessary bcos some
//servlet engines are slow to notice the end of stream







